@@ -40,6 +40,9 @@
 #include <QTimer>
 #include <QSessionManager>
 #include <QClipboard>
+#include <QDBusConnection>
+#include <QDBusInterface>
+#include <QDBusPendingCall>
 
 // KDE includes
 #include <KAboutApplicationDialog>
@@ -138,7 +141,8 @@ DiscoverObject::DiscoverObject(CompactMode mode)
     auto action = new OneTimeAction(
         [this]() {
             bool found = DiscoverBackendsFactory::hasRequestedBackends();
-            for (auto b : ResourcesModel::global()->backends())
+            const auto backends = ResourcesModel::global()->backends();
+            for (auto b : backends)
                 found |= b->hasApplications();
 
             if (!found)
@@ -239,8 +243,7 @@ void DiscoverObject::openLocalPackage(const QUrl& localfile)
             AbstractResourcesBackend::Filters f;
             f.resourceUrl = localfile;
             auto stream = new StoredResultsStream({ResourcesModel::global()->search(f)});
-            connect(stream, &StoredResultsStream::finished, this, [this, localfile, stream]() {
-                const auto res = stream->resources();
+            connect(stream, &StoredResultsStream::finishedResources, this, [this, localfile](const QVector<AbstractResource*> &res) {
                 if (res.count() == 1) {
                     emit openApplicationInternal(res.first());
                 } else {
@@ -275,8 +278,7 @@ void DiscoverObject::openApplication(const QUrl& url)
             AbstractResourcesBackend::Filters f;
             f.resourceUrl = url;
             auto stream = new StoredResultsStream({ResourcesModel::global()->search(f)});
-            connect(stream, &StoredResultsStream::finished, this, [this, url, stream]() {
-                const auto res = stream->resources();
+            connect(stream, &StoredResultsStream::finishedResources, this, [this, url](const QVector<AbstractResource*> &res) {
                 if (res.count() >= 1) {
                     emit openApplicationInternal(res.first());
                 } else {
@@ -486,5 +488,16 @@ void DiscoverObject::copyTextToClipboard(const QString& text)
     qGuiApp->clipboard()->setText(text);
 }
 
+void DiscoverObject::reboot()
+{
+    QDBusInterface interface(QStringLiteral("org.kde.ksmserver"), QStringLiteral("/KSMServer"), QStringLiteral("org.kde.KSMServerInterface"), QDBusConnection::sessionBus());
+    interface.asyncCall(QStringLiteral("logout"), 0, 1, 2); // Options: do not ask again | reboot | force
+}
+
+QRect DiscoverObject::initialGeometry() const
+{
+    KConfigGroup window(KSharedConfig::openConfig(), "Window");
+    return window.readEntry("geometry", QRect());
+}
 
 #include "DiscoverObject.moc"
