@@ -20,7 +20,7 @@
 
 #include "ScreenshotsModel.h"
 #include <resources/AbstractResource.h>
-#include <QDebug>
+#include "libdiscover_debug.h"
 // #include <tests/modeltest.h>
 
 ScreenshotsModel::ScreenshotsModel(QObject* parent)
@@ -45,12 +45,13 @@ void ScreenshotsModel::setResource(AbstractResource* res)
         disconnect(m_resource, &AbstractResource::screenshotsFetched, this, &ScreenshotsModel::screenshotsFetched);
     }
     m_resource = res;
+    Q_EMIT resourceChanged(res);
     
     if(res) {
         connect(m_resource, &AbstractResource::screenshotsFetched, this, &ScreenshotsModel::screenshotsFetched);
         res->fetchScreenshots();
     } else
-        qWarning() << "empty resource!";
+        qCWarning(LIBDISCOVER_LOG) << "empty resource!";
 }
 
 AbstractResource* ScreenshotsModel::resource() const
