@@ -20,6 +20,7 @@
  ***************************************************************************/
 
 #include "OdrsReviewsBackend.h"
+#include "AppStreamIntegration.h"
 
 #include <ReviewsBackend/Review.h>
 #include <ReviewsBackend/Rating.h>
@@ -91,32 +92,7 @@ void OdrsReviewsBackend::ratingsFetched(KJob *job)
 
 static QString osName()
 {
-    QString osReleaseFilename;
-    if (QFileInfo::exists(QStringLiteral("/etc/os-release"))) {
-        osReleaseFilename = QStringLiteral("/etc/os-release");
-    } else if (QFileInfo::exists(QStringLiteral("/usr/lib/os-release"))) {
-        osReleaseFilename = QStringLiteral("/usr/lib/os-release");
-    }
-
-    if (osReleaseFilename.isEmpty()) {
-        return QStringLiteral("Unknown");
-    }
-
-    QFile osReleaseFile(osReleaseFilename);
-    if (osReleaseFile.open(QIODevice::ReadOnly)) {
-        QString line;
-        QTextStream stream(&osReleaseFile);
-        while (stream.readLineInto(&line)) {
-            if (line.startsWith(QStringLiteral("NAME"))) {
-                QStringRef name = line.midRef(5).trimmed();
-                if (name.startsWith(QLatin1Char('\"')) && name.endsWith(QLatin1Char('\"')))
-                    name = name.mid(1, name.size()-2);
-                return name.toString();
-            }
-        }
-    }
-
-    return QStringLiteral("Unknown");
+    return AppStreamIntegration::global()->osRelease()->name();
 }
 
 static QString userHash()
@@ -179,7 +155,7 @@ void OdrsReviewsBackend::fetchReviews(AbstractResource *app, int page)
 void OdrsReviewsBackend::reviewsFetched()
 {
     QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
-
+    QScopedPointer<QNetworkReply, QScopedPointerDeleteLater> replyPtr(reply);
     if (reply->error() != QNetworkReply::NoError) {
         qCWarning(LIBDISCOVER_LOG) << "error fetching reviews:" << reply->errorString();
         m_isFetching = false;
@@ -191,7 +167,6 @@ void OdrsReviewsBackend::reviewsFetched()
     AbstractResource *resource = qobject_cast<AbstractResource*>(reply->request().originatingObject());
     Q_ASSERT(resource);
     parseReviews(document, resource);
-
     // Store reviews to cache so we don't need to download them all the time
     if (document.array().isEmpty()) {
         return;
@@ -244,6 +219,7 @@ void OdrsReviewsBackend::usefulnessSubmitted()
     } else {
         qCWarning(LIBDISCOVER_LOG) << "Failed to submit usefulness: " << reply->errorString();
     }
+    reply->deleteLater();
 }
 
 QString OdrsReviewsBackend::userName() const
@@ -295,6 +271,7 @@ void OdrsReviewsBackend::reviewSubmitted(QNetworkReply *reply)
     } else {
         qCWarning(LIBDISCOVER_LOG) << "Failed to submit review: " << reply->errorString();
     }
+    reply->deleteLater();
 }
 
 void OdrsReviewsBackend::parseRatings()
