@@ -59,6 +59,8 @@
 #include <glib.h>
 #include <QRegularExpression>
 
+#include <sys/stat.h>
+
 DISCOVER_BACKEND_PLUGIN(FlatpakBackend)
 
 QDebug operator<<(QDebug debug, const FlatpakResource::Id& id)
@@ -73,10 +75,10 @@ QDebug operator<<(QDebug debug, const FlatpakResource::Id& id)
     return debug;
 }
 
-static FlatpakResource::Id idForInstalledRef(FlatpakInstallation *installation, FlatpakInstalledRef *ref)
+static FlatpakResource::Id idForInstalledRef(FlatpakInstallation *installation, FlatpakInstalledRef *ref, const QString &postfix)
 {
     const FlatpakResource::ResourceType appType = flatpak_ref_get_kind(FLATPAK_REF(ref)) == FLATPAK_REF_KIND_APP ? FlatpakResource::DesktopApp : FlatpakResource::Runtime;
-    const QString appId = QLatin1String(flatpak_ref_get_name(FLATPAK_REF(ref)));
+    const QString appId = QLatin1String(flatpak_ref_get_name(FLATPAK_REF(ref))) + postfix;
     const QString arch = QString::fromUtf8(flatpak_ref_get_arch(FLATPAK_REF(ref)));
     const QString branch = QString::fromUtf8(flatpak_ref_get_branch(FLATPAK_REF(ref)));
 
@@ -106,6 +108,13 @@ FlatpakBackend::FlatpakBackend(QObject* parent)
     }
 
     connect(m_reviews.data(), &OdrsReviewsBackend::ratingsReady, this, &FlatpakBackend::announceRatingsReady);
+
+    /* Override the umask to 022 to make it possible to share files between
+     * the plasma-discover process and flatpak system helper process.
+     *
+     * See https://github.com/flatpak/flatpak/pull/2856/
+     */
+    umask(022);
 }
 
 FlatpakBackend::~FlatpakBackend()
@@ -113,7 +122,7 @@ FlatpakBackend::~FlatpakBackend()
     g_cancellable_cancel(m_cancellable);
     m_threadPool.waitForDone(200);
     m_threadPool.clear();
-    for(auto inst : m_installations)
+    for(auto inst : qAsConst(m_installations))
         g_object_unref(inst);
 
     g_object_unref(m_cancellable);
@@ -131,7 +140,7 @@ void FlatpakBackend::announceRatingsReady()
     const auto ids = m_reviews->appstreamIds().toSet();
     foreach(AbstractResource* res, m_resources) {
         if (ids.contains(res->appstreamId())) {
-            res->ratingFetched();
+            Q_EMIT res->ratingFetched();
         }
     }
 }
@@ -149,9 +158,11 @@ public:
 
     void start()
     {
-        auto replyGet = get(QNetworkRequest(m_url));
-
+        QNetworkRequest req(m_url);
+        req.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
+        auto replyGet = get(req);
         connect(replyGet, &QNetworkReply::finished, this, [this, replyGet] {
+            QScopedPointer<QNetworkReply, QScopedPointerDeleteLater> replyPtr(replyGet);
             const QUrl originalUrl = replyGet->request().url();
             if (replyGet->error() != QNetworkReply::NoError) {
                 qWarning() << "couldn't download" << originalUrl << replyGet->errorString();
@@ -162,6 +173,7 @@ public:
             const QUrl fileUrl = QUrl::fromLocalFile(QStandardPaths::writableLocation(QStandardPaths::TempLocation) + QLatin1Char('/') + originalUrl.fileName());
             auto replyPut = put(QNetworkRequest(fileUrl), replyGet->readAll());
             connect(replyPut, &QNetworkReply::finished, this, [this, originalUrl, fileUrl, replyPut]() {
+                QScopedPointer<QNetworkReply, QScopedPointerDeleteLater> replyPtr(replyPut);
                 if (replyPut->error() != QNetworkReply::NoError) {
                     qWarning() << "couldn't save" << originalUrl << replyPut->errorString();
                     Q_EMIT jobFinished(false, nullptr);
@@ -240,7 +252,14 @@ FlatpakInstalledRef * FlatpakBackend::getInstalledRefForApp(FlatpakInstallation
 
 FlatpakResource * FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *flatpakInstallation, FlatpakInstalledRef *ref) const
 {
-    return m_resources.value(idForInstalledRef(flatpakInstallation, ref));
+    auto r = m_resources.value(idForInstalledRef(flatpakInstallation, ref, {}));
+    if (!r)
+        r = m_resources.value(idForInstalledRef(flatpakInstallation, ref, QStringLiteral(".desktop")));
+
+    if (!r) {
+        qDebug() << "no" << flatpak_ref_get_name(FLATPAK_REF(ref));
+    }
+    return r;
 }
 
 FlatpakResource * FlatpakBackend::getRuntimeForApp(FlatpakResource *resource) const
@@ -626,13 +645,22 @@ bool FlatpakBackend::loadAppsFromAppstreamData(FlatpakInstallation *flatpakInsta
     return true;
 }
 
+void FlatpakBackend::metadataRefreshed()
+{
+    m_refreshAppstreamMetadataJobs--;
+    if (m_refreshAppstreamMetadataJobs == 0) {
+        loadInstalledApps();
+        checkForUpdates();
+    }
+}
+
 void FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote)
 {
     Q_ASSERT(m_refreshAppstreamMetadataJobs != 0);
 
     FlatpakSource source(remote);
     if (!source.isEnabled() || flatpak_remote_get_noenumerate(remote)) {
-        m_refreshAppstreamMetadataJobs--;
+        metadataRefreshed();
         return;
     }
 
@@ -640,8 +668,8 @@ void FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, F
     const QString appstreamIconsPath = source.appstreamDir() + QLatin1String("/icons/");
     const QString appDirFileName = appstreamDirPath + QLatin1String("/appstream.xml.gz");
     if (!QFile::exists(appDirFileName)) {
-        m_refreshAppstreamMetadataJobs--;
         qWarning() << "No" << appDirFileName << "appstream metadata found for" << source.name();
+        metadataRefreshed();
         return;
     }
 
@@ -656,12 +684,7 @@ void FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, F
             addResource(resource);
         }
 
-        m_refreshAppstreamMetadataJobs--;
-        if (m_refreshAppstreamMetadataJobs == 0) {
-            loadInstalledApps();
-            checkForUpdates();
-
-        }
+        metadataRefreshed();
         acquireFetching(false);
         fw->deleteLater();
     });
@@ -706,25 +729,31 @@ bool FlatpakBackend::loadInstalledApps(FlatpakInstallation *flatpakInstallation)
     for (uint i = 0; i < refs->len; i++) {
         FlatpakInstalledRef *ref = FLATPAK_INSTALLED_REF(g_ptr_array_index(refs, i));
 
-        if (flatpak_ref_get_kind(FLATPAK_REF(ref)) == FLATPAK_REF_KIND_RUNTIME) {
+        const auto name = QLatin1String(flatpak_ref_get_name(FLATPAK_REF(ref)));
+        if (name.endsWith(QLatin1String(".Debug")) || name.endsWith(QLatin1String(".Locale")) || name.endsWith(QLatin1String(".BaseApp")) || name.endsWith(QLatin1String(".Docs")))
             continue;
-        }
+
         const auto res = getAppForInstalledRef(flatpakInstallation, ref);
         if (res) {
             res->setState(AbstractResource::Installed);
             continue;
         }
 
-        const auto name = QLatin1String(flatpak_ref_get_name(FLATPAK_REF(ref)));
+        AppStream::Component cid;
         AppStream::Metadata metadata;
         const QString fnDesktop = pathApps + name + QLatin1String(".desktop");
         AppStream::Metadata::MetadataError error = metadata.parseFile(fnDesktop, AppStream::Metadata::FormatKindDesktopEntry);
         if (error != AppStream::Metadata::MetadataErrorNoError) {
-            qWarning() << "Failed to parse appstream metadata: " << error << fnDesktop;
-            continue;
-        }
+            qDebug() << "Failed to parse appstream metadata:" << error << fnDesktop;
 
-        FlatpakResource *resource = new FlatpakResource(metadata.component(), flatpakInstallation, this);
+            cid.setId(QString::fromLatin1(flatpak_ref_get_name(FLATPAK_REF(ref))));
+#if FLATPAK_CHECK_VERSION(1,1,2)
+            cid.setName(QString::fromLatin1(flatpak_installed_ref_get_appdata_name(ref)));
+#endif
+        } else
+            cid = metadata.component();
+
+        FlatpakResource *resource = new FlatpakResource(cid, flatpakInstallation, this);
 
         resource->setIconPath(pathExports);
         resource->setState(AbstractResource::Installed);
@@ -883,10 +912,11 @@ private:
 void FlatpakBackend::refreshAppstreamMetadata(FlatpakInstallation *installation, FlatpakRemote *remote)
 {
     FlatpakRefreshAppstreamMetadataJob *job = new FlatpakRefreshAppstreamMetadataJob(installation, remote);
-    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFailed, this, [this] () {
-        m_refreshAppstreamMetadataJobs--;
-    });
+    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFailed, this, &FlatpakBackend::metadataRefreshed);
     connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFinished, this, &FlatpakBackend::integrateRemote);
+    connect(job, &FlatpakRefreshAppstreamMetadataJob::finished, this, [this] { acquireFetching(false); });
+
+    acquireFetching(true);
     job->start();
 }
 
@@ -1126,7 +1156,7 @@ ResultsStream * FlatpakBackend::search(const AbstractResourcesBackend::Filters &
         FlatpakFetchRemoteResourceJob *fetchResourceJob = new FlatpakFetchRemoteResourceJob(filter.resourceUrl, this);
         connect(fetchResourceJob, &FlatpakFetchRemoteResourceJob::jobFinished, this, [fetchResourceJob, stream] (bool success, FlatpakResource *resource) {
             if (success) {
-                stream->resourcesFound({resource});
+                Q_EMIT stream->resourcesFound({resource});
             }
             stream->finish();
             fetchResourceJob->deleteLater();
@@ -1143,17 +1173,18 @@ ResultsStream * FlatpakBackend::search(const AbstractResourcesBackend::Filters &
     auto f = [this, stream, filter] () {
         QVector<AbstractResource*> ret;
         foreach(auto r, m_resources) {
-            if (r->type() == AbstractResource::Technical && filter.state != AbstractResource::Upgradeable) {
+            const bool matchById = filter.search.compare(r->appstreamId(), Qt::CaseInsensitive) == 0;
+            if (r->type() == AbstractResource::Technical && filter.state != AbstractResource::Upgradeable && !matchById) {
                 continue;
             }
-
             if (r->state() < filter.state)
                 continue;
 
             if (!filter.extends.isEmpty() && !r->extends().contains(filter.extends))
                 continue;
 
-            if (filter.search.isEmpty() || r->name().contains(filter.search, Qt::CaseInsensitive) || r->comment().contains(filter.search, Qt::CaseInsensitive)) {
+            if (filter.search.isEmpty() || r->name().contains(filter.search, Qt::CaseInsensitive) || r->comment().contains(filter.search, Qt::CaseInsensitive) || matchById)
+            {
                 ret += r;
             }
         }
@@ -1187,7 +1218,7 @@ ResultsStream * FlatpakBackend::findResourceByPackageName(const QUrl &url)
 {
     if (url.scheme() == QLatin1String("appstream")) {
         if (url.host().isEmpty())
-            passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
+            Q_EMIT passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
         else {
             auto stream = new ResultsStream(QStringLiteral("FlatpakStream"));
             auto f = [this, stream, url] () {
@@ -1238,7 +1269,7 @@ Transaction* FlatpakBackend::installApplication(AbstractResource *app, const Add
     }
 
     FlatpakJobTransaction *transaction = new FlatpakJobTransaction(resource, Transaction::InstallRole);
-    connect(transaction, &FlatpakJobTransaction::statusChanged, [this, resource] (Transaction::Status status) {
+    connect(transaction, &FlatpakJobTransaction::statusChanged, this, [this, resource] (Transaction::Status status) {
         if (status == Transaction::Status::DoneStatus) {
             FlatpakInstallation *installation = resource->installation();
             updateAppState(installation, resource);
@@ -1267,7 +1298,7 @@ Transaction* FlatpakBackend::removeApplication(AbstractResource *app)
     FlatpakInstallation *installation = resource->installation();
     FlatpakJobTransaction *transaction = new FlatpakJobTransaction(resource, Transaction::RemoveRole);
 
-    connect(transaction, &FlatpakJobTransaction::statusChanged, [this, installation, resource] (Transaction::Status status) {
+    connect(transaction, &FlatpakJobTransaction::statusChanged, this, [this, installation, resource] (Transaction::Status status) {
         if (status == Transaction::Status::DoneStatus) {
             updateAppSize(installation, resource);
         }
