@@ -65,7 +65,12 @@ class KNSBackendFactory : public AbstractResourcesBackendFactory {
         QVector<AbstractResourcesBackend*> newInstance(QObject* parent, const QString &/*name*/) const override
         {
             QVector<AbstractResourcesBackend*> ret;
-            for (const QString &path: QStandardPaths::standardLocations(QStandardPaths::GenericConfigLocation)) {
+#if KNEWSTUFFCORE_VERSION_MAJOR==5 && KNEWSTUFFCORE_VERSION_MINOR>=57
+            QStringList locations = KNSCore::Engine::configSearchLocations();
+#else
+            QStringList locations = QStandardPaths::standardLocations(QStandardPaths::GenericConfigLocation);
+#endif
+            for (const QString &path: locations) {
                 QDirIterator dirIt(path, {QStringLiteral("*.knsrc")}, QDir::Files);
                 for(; dirIt.hasNext(); ) {
                     dirIt.next();
@@ -110,15 +115,6 @@ KNSBackend::KNSBackend(QObject* parent, const QString& iconName, const QString &
 
     setFetching(true);
 
-    m_engine = new KNSCore::Engine(this);
-    m_engine->init(m_name);
-    m_engine->setPageSize(100);
-    connect(m_engine, &KNSCore::Engine::signalErrorCode, this, &KNSBackend::signalErrorCode);
-    connect(m_engine, &KNSCore::Engine::signalEntriesLoaded, this, &KNSBackend::receivedEntries, Qt::QueuedConnection);
-    connect(m_engine, &KNSCore::Engine::signalEntryChanged, this, &KNSBackend::statusChanged, Qt::QueuedConnection);
-    connect(m_engine, &KNSCore::Engine::signalEntryDetailsLoaded, this, &KNSBackend::statusChanged);
-    connect(m_engine, &KNSCore::Engine::signalProvidersLoaded, this, &KNSBackend::fetchInstalled);
-
     // This ensures we have something to track when checking after the initialization timeout
     connect(this, &KNSBackend::initialized, this, [this](){ m_initialized = true; });
     // If we have not initialized in 60 seconds, consider this KNS backend invalid
@@ -139,22 +135,83 @@ KNSBackend::KNSBackend(QObject* parent, const QString& iconName, const QString &
         m_displayName[0] = m_displayName[0].toUpper();
     }
 
-    static const QSet<QString> knsrcPlasma = {
-        QStringLiteral("aurorae.knsrc"), QStringLiteral("icons.knsrc"), QStringLiteral("kfontinst.knsrc"), QStringLiteral("lookandfeel.knsrc"), QStringLiteral("plasma-themes.knsrc"), QStringLiteral("plasmoids.knsrc"),
-        QStringLiteral("wallpaper.knsrc"), QStringLiteral("xcursor.knsrc"),
+    const QStringList cats = group.readEntry<QStringList>("Categories", QStringList{});
+    QVector<Category*> categories;
+    if (cats.count() > 1) {
+        m_categories += cats;
+        for(const auto &cat: cats)
+            categories << new Category(cat, {}, { {CategoryFilter, cat } }, backendName, {}, {}, true);
+    }
 
-        QStringLiteral("cgcgtk3.knsrc"), QStringLiteral("cgcicon.knsrc"), QStringLiteral("cgctheme.knsrc"), //GTK integration
-        QStringLiteral("kwinswitcher.knsrc"), QStringLiteral("kwineffect.knsrc"), QStringLiteral("kwinscripts.knsrc"), //KWin
-        QStringLiteral("comic.knsrc"), QStringLiteral("colorschemes.knsrc"),
-        QStringLiteral("emoticons.knsrc"), QStringLiteral("plymouth.knsrc"),
-        QStringLiteral("sddmtheme.knsrc"), QStringLiteral("wallpaperplugin.knsrc")
-    };
-    auto actualCategory = new Category(m_displayName, QStringLiteral("plasma"), filters, backendName, {}, QUrl(), true);
+    QVector<Category*> topCategories{categories};
+    for (const auto &cat: categories) {
+        const QString catName = cat->name().append(QLatin1Char('/'));
+        for (const auto& potentialSubCat: categories) {
+            if(potentialSubCat->name().startsWith(catName)) {
+                cat->addSubcategory(potentialSubCat);
+                topCategories.removeOne(potentialSubCat);
+            }
+        }
+    }
 
-    const auto topLevelName = knsrcPlasma.contains(fileName)? i18n("Plasma Addons") : i18n("Application Addons");
-    const QUrl decoration(knsrcPlasma.contains(fileName)? QStringLiteral("https://c2.staticflickr.com/4/3148/3042248532_20bd2e38f4_b.jpg") : QStringLiteral("https://c2.staticflickr.com/8/7067/6847903539_d9324dcd19_o.jpg"));
-    auto addonsCategory = new Category(topLevelName, QStringLiteral("plasma"), filters, backendName, {actualCategory}, decoration, true);
-    m_rootCategories = { addonsCategory };
+    m_engine = new KNSCore::Engine(this);
+    connect(m_engine, &KNSCore::Engine::signalErrorCode, this, &KNSBackend::signalErrorCode);
+    connect(m_engine, &KNSCore::Engine::signalEntriesLoaded, this, &KNSBackend::receivedEntries, Qt::QueuedConnection);
+    connect(m_engine, &KNSCore::Engine::signalEntryChanged, this, &KNSBackend::statusChanged, Qt::QueuedConnection);
+    connect(m_engine, &KNSCore::Engine::signalEntryDetailsLoaded, this, &KNSBackend::statusChanged);
+    connect(m_engine, &KNSCore::Engine::signalProvidersLoaded, this, &KNSBackend::fetchInstalled);
+    connect(m_engine, &KNSCore::Engine::signalCategoriesMetadataLoded, this, [categories](const QList< KNSCore::Provider::CategoryMetadata>& categoryMetadatas){
+        for (const KNSCore::Provider::CategoryMetadata& category : categoryMetadatas) {
+            for (Category* cat : categories) {
+                if (cat->orFilters().count() > 0 && cat->orFilters().first().second == category.name) {
+                    cat->setName(category.displayName);
+                    break;
+                }
+            }
+        }
+    });
+    m_engine->setPageSize(100);
+    m_engine->init(m_name);
+
+    m_hasApplications = group.readEntry<bool>("X-Discover-HasApplications", false);
+    if(m_hasApplications) {
+        auto actualCategory = new Category(m_displayName, QStringLiteral("plasma"), filters, backendName, topCategories, QUrl(), false);
+        auto applicationCategory = new Category(i18n("Applications"), QStringLiteral("applications-internet"), filters, backendName, { actualCategory }, QUrl(), false);
+        applicationCategory->setAndFilter({ {CategoryFilter, QLatin1String("Application")} });
+        m_categories.append(applicationCategory->name());
+        m_rootCategories = { applicationCategory };
+        // Make sure we filter out any apps which won't run on the current system architecture
+        QStringList tagFilter = m_engine->tagFilter();
+        if(QSysInfo::currentCpuArchitecture() == QLatin1String("arm")) {
+            tagFilter << QLatin1String("application##architecture==armhf");
+        } else if(QSysInfo::currentCpuArchitecture() == QLatin1String("arm64")) {
+            tagFilter << QLatin1String("application##architecture==arm64");
+        } else if(QSysInfo::currentCpuArchitecture() == QLatin1String("i386")) {
+            tagFilter << QLatin1String("application##architecture==x86");
+        } else if(QSysInfo::currentCpuArchitecture() == QLatin1String("ia64")) {
+            tagFilter << QLatin1String("application##architecture==x86-64");
+        } else if(QSysInfo::currentCpuArchitecture() == QLatin1String("x86_64")) {
+            tagFilter << QLatin1String("application##architecture==x86");
+            tagFilter << QLatin1String("application##architecture==x86-64");
+        }
+        m_engine->setTagFilter(tagFilter);
+    } else {
+        static const QSet<QString> knsrcPlasma = {
+            QStringLiteral("aurorae.knsrc"), QStringLiteral("icons.knsrc"), QStringLiteral("kfontinst.knsrc"), QStringLiteral("lookandfeel.knsrc"), QStringLiteral("plasma-themes.knsrc"), QStringLiteral("plasmoids.knsrc"),
+            QStringLiteral("wallpaper.knsrc"), QStringLiteral("xcursor.knsrc"),
+
+            QStringLiteral("cgcgtk3.knsrc"), QStringLiteral("cgcicon.knsrc"), QStringLiteral("cgctheme.knsrc"), //GTK integration
+            QStringLiteral("kwinswitcher.knsrc"), QStringLiteral("kwineffect.knsrc"), QStringLiteral("kwinscripts.knsrc"), //KWin
+            QStringLiteral("comic.knsrc"), QStringLiteral("colorschemes.knsrc"),
+            QStringLiteral("emoticons.knsrc"), QStringLiteral("plymouth.knsrc"),
+            QStringLiteral("sddmtheme.knsrc"), QStringLiteral("wallpaperplugin.knsrc")
+        };
+        auto actualCategory = new Category(m_displayName, QStringLiteral("plasma"), filters, backendName, categories, QUrl(), true);
+
+        const auto topLevelName = knsrcPlasma.contains(fileName)? i18n("Plasma Addons") : i18n("Application Addons");
+        auto addonsCategory = new Category(topLevelName, QStringLiteral("plasma"), filters, backendName, {actualCategory}, QUrl(), true);
+        m_rootCategories = { addonsCategory };
+    }
 }
 
 KNSBackend::~KNSBackend()
@@ -209,7 +266,16 @@ KNSResource* KNSBackend::resourceForEntry(const KNSCore::EntryInternal& entry)
 {
     KNSResource* r = static_cast<KNSResource*>(m_resourcesByName.value(entry.uniqueId()));
     if (!r) {
-        r = new KNSResource(entry, m_categories, this);
+        QStringList categories{name(), m_rootCategories.first()->name()};
+        const auto cats = m_engine->categoriesMetadata();
+        const int catIndex = kIndexOf(cats, [&entry](const KNSCore::Provider::CategoryMetadata& cat){ return entry.category() == cat.id; });
+        if (catIndex > -1) {
+            categories << cats.at(catIndex).name;
+        }
+        if(m_hasApplications) {
+            categories << QLatin1String("Application");
+        }
+        r = new KNSResource(entry, categories, this);
         m_resourcesByName.insert(entry.uniqueId(), r);
     } else {
         r->setEntry(entry);
@@ -338,7 +404,8 @@ public:
             };
         else
             actionFunction = [res, engine]() {
-                for(auto i : res->linkIds())
+                const auto links = res->linkIds();
+                for(auto i : links)
                     engine->install(res->entry(), i);
             };
         QTimer::singleShot(0, res, actionFunction);
@@ -431,7 +498,9 @@ ResultsStream* KNSBackend::search(const AbstractResourcesBackend::Filters& filte
         }
 
         return stream;
-    } else if (filter.category && filter.category->matchesCategoryName(m_categories.constFirst())) {
+    } else if ((m_hasApplications && !filter.category) // If there is no category defined, we are searching in the root, and should include only application results
+              // If there /is/ a category, make sure we actually are one of those requested before searching
+           || (filter.category && kContains(m_categories, [&filter](const QString& cat) { return filter.category->matchesCategoryName(cat); }))) {
         auto r = new ResultsStream(QStringLiteral("KNS-search-")+name());
         searchStream(r, filter.search);
         return r;
