@@ -141,9 +141,9 @@ KNSCore::EntryInternal KNSResource::entry() const
     return m_entry;
 }
 
-QString KNSResource::license()
+QJsonArray KNSResource::licenses()
 {
-    return m_entry.license();
+    return { QJsonObject{ {QStringLiteral("name"), m_entry.license()} } };
 }
 
 int KNSResource::size()
@@ -234,6 +234,9 @@ void KNSResource::invokeApplication() const
 
 QString KNSResource::executeLabel() const
 {
+    if(knsBackend()->hasApplications()) {
+        return i18n("Launch");
+    }
     return i18n("Use");
 }
 
@@ -245,7 +248,8 @@ QDate KNSResource::releaseDate() const
 QVector<int> KNSResource::linkIds() const
 {
     QVector<int> ids;
-    for(const auto &e : m_entry.downloadLinkInformationList()) {
+    const auto linkInfo = m_entry.downloadLinkInformationList();
+    for(const auto &e : linkInfo) {
         if (e.isDownloadtypeLink)
             ids << e.id;
     }
