@@ -332,7 +332,8 @@ void PackageKitBackend::includePackagesToAdd()
             auto res = m_packages.packages.take(pkg);
             if (res) {
                 if (AppPackageKitResource* ares = qobject_cast<AppPackageKitResource*>(res)) {
-                    for(const auto &ext: res->extends())
+                    const auto extends = res->extends();
+                    for(const auto &ext: extends)
                         m_packages.extendedBy[ext].removeAll(ares);
                 }
 
@@ -418,13 +419,13 @@ ResultsStream* PackageKitBackend::search(const AbstractResourcesBackend::Filters
         const auto ext = kTransform<QVector<AbstractResource*>>(m_packages.extendedBy[filter.extends], [](AppPackageKitResource* a){ return a; });
         return new ResultsStream(QStringLiteral("PackageKitStream-extends"), ext);
     } else if (filter.search.isEmpty()) {
-        return new ResultsStream(QStringLiteral("PackageKitStream-all"), kFilter<QVector<AbstractResource*>>(m_packages.packages, [](AbstractResource* res) { return res->type() != AbstractResource::Technical; }));
+        return new ResultsStream(QStringLiteral("PackageKitStream-all"), kFilter<QVector<AbstractResource*>>(m_packages.packages, [](AbstractResource* res) { return res->type() != AbstractResource::Technical && !qobject_cast<PackageKitResource*>(res)->extendsItself(); }));
     } else {
         const QList<AppStream::Component> components = m_appdata->search(filter.search);
         const QStringList ids = kTransform<QStringList>(components, [](const AppStream::Component& comp) { return comp.id(); });
         auto stream = new ResultsStream(QStringLiteral("PackageKitStream-search"));
         if (!ids.isEmpty()) {
-            const auto resources = resourcesByPackageNames<QVector<AbstractResource*>>(ids);
+            const auto resources = kFilter<QVector<AbstractResource*>>(resourcesByPackageNames<QVector<AbstractResource*>>(ids), [](AbstractResource* res){ return !qobject_cast<PackageKitResource*>(res)->extendsItself(); });
             QTimer::singleShot(0, this, [stream, resources] () {
                 Q_EMIT stream->resourcesFound(resources);
             });
@@ -489,8 +490,8 @@ ResultsStream * PackageKitBackend::findResourceByPackageName(const QUrl& url)
                     break;
                 }
             }
-            if (!pkg)
-                qDebug() << "could not find" << host << deprecatedHost;
+//             if (!pkg)
+//                 qDebug() << "could not find" << host << deprecatedHost;
         }
     }
     return new ResultsStream(QStringLiteral("PackageKitStream-url"), pkg ? QVector<AbstractResource*>{pkg} : QVector<AbstractResource*>{});
@@ -508,7 +509,8 @@ int PackageKitBackend::updatesCount() const
 
     int ret = 0;
     QSet<QString> packages;
-    for(auto res: upgradeablePackages()) {
+    const auto toUpgrade = upgradeablePackages();
+    for(auto res: toUpgrade) {
         const auto packageName = res->packageName();
         if (packages.contains(packageName)) {
             continue;
@@ -652,41 +654,9 @@ AbstractReviewsBackend* PackageKitBackend::reviewsBackend() const
     return m_reviews.data();
 }
 
-static QString readDistroName()
-{
-    const QStringList osreleasenames = (QStringList() << QStringLiteral("/etc/os-release")
-                                                      << QStringLiteral("/usr/lib/os-release"));
-    foreach (QString osrelease, osreleasenames)
-    {
-        QFile file(osrelease);
-        if (file.open(QIODevice::ReadOnly | QIODevice::Text))
-        {
-            QByteArray line;
-            while (!file.atEnd()) {
-                line = file.readLine().trimmed();
-                if (line.startsWith("NAME=")) {
-                    auto output = line.right(line.length()-5);
-                    output = output.replace('\"',"");
-                    return QString::fromLocal8Bit(output);
-                }
-            }
-        }
-    }
-
-    QProcess process;
-    process.setEnvironment({QStringLiteral("LC_ALL=C")});
-    process.start(QStringLiteral("lsb_release"), {QStringLiteral("-sd")});
-    process.waitForFinished();
-    auto output = process.readAll().trimmed();
-    if (output.startsWith('\"') && output.endsWith('\"'))
-        output = output.mid(1, output.length()-2);
-    return QString::fromLocal8Bit(output);
-}
-
 QString PackageKitBackend::displayName() const
 {
-    static const QString distro = readDistroName();
-    return distro;
+    return AppStreamIntegration::global()->osRelease()->prettyName();
 }
 
 #include "PackageKitBackend.moc"
