@@ -102,10 +102,15 @@ QVariant PackageKitResource::icon() const
     return QStringLiteral("applications-other");
 }
 
-QString PackageKitResource::license()
+QJsonArray PackageKitResource::licenses()
 {
     fetchDetails();
-    return m_details.license().isEmpty() ? i18n("Unknown") : m_details.license();
+
+    if (!m_details.license().isEmpty()) {
+        return {{ m_details.license(), {} }};
+    }
+
+    return {};
 }
 
 QList<PackageState> PackageKitResource::addonsInformation()
@@ -256,7 +261,7 @@ static QStringList urlToLinks(const QStringList& urls)
     return ret;
 }
 
-void PackageKitResource::updateDetail(const QString& packageID, const QStringList& updates, const QStringList& obsoletes, const QStringList& vendorUrls,
+void PackageKitResource::updateDetail(const QString& packageID, const QStringList& /*updates*/, const QStringList& obsoletes, const QStringList& vendorUrls,
                                       const QStringList& /*bugzillaUrls*/, const QStringList& /*cveUrls*/, PackageKit::Transaction::Restart restart, const QString &_updateText,
                                       const QString& /*changelog*/, PackageKit::Transaction::UpdateState state, const QDateTime& /*issued*/, const QDateTime& /*updated*/)
 {
@@ -334,3 +339,18 @@ void PackageKitResource::fetchDependencies()
         Q_EMIT dependenciesFound(*packageDependencies);
     });
 }
+
+bool PackageKitResource::extendsItself() const
+{
+    const auto extendsResources = backend()->resourcesByPackageNames<QVector<AbstractResource*>>(extends());
+    if (extendsResources.isEmpty())
+        return false;
+
+    const auto ourPackageNames = allPackageNames();
+    for (auto r : extendsResources) {
+        PackageKitResource* pkres = qobject_cast<PackageKitResource*>(r);
+        if (pkres->allPackageNames() != ourPackageNames)
+            return false;
+    }
+    return true;
+}
