@@ -53,6 +53,7 @@ ResourcesModel::ResourcesModel(QObject* parent, bool load)
     , m_isFetching(false)
     , m_initializingBackends(0)
     , m_currentApplicationBackend(nullptr)
+    , m_allInitializedEmitter(new QTimer(this))
 {
     init(load);
     connect(this, &ResourcesModel::allInitialized, this, &ResourcesModel::slotFetching);
@@ -64,6 +65,13 @@ void ResourcesModel::init(bool load)
     Q_ASSERT(!s_self);
     Q_ASSERT(QCoreApplication::instance()->thread()==QThread::currentThread());
 
+    m_allInitializedEmitter->setSingleShot(true);
+    m_allInitializedEmitter->setInterval(0);
+    connect(m_allInitializedEmitter, &QTimer::timeout, this, [this](){
+        if (m_initializingBackends == 0)
+            emit allInitialized();
+    });
+
     if(load)
         QMetaObject::invokeMethod(this, "registerAllBackends", Qt::QueuedConnection);
 
@@ -119,10 +127,15 @@ void ResourcesModel::addResourcesBackend(AbstractResourcesBackend* backend)
     connect(backend, &AbstractResourcesBackend::passiveMessage, this, &ResourcesModel::passiveMessage);
     connect(backend->backendUpdater(), &AbstractBackendUpdater::progressingChanged, this, &ResourcesModel::slotFetching);
 
-    if(m_initializingBackends==0)
-        emit allInitialized();
-    else
+    // In case this is in fact the first backend to be added, and also happens to be
+    // pre-filled, we still need for the rest of the backends to be added before trying
+    // to send out the initialized signal. To ensure this happens, schedule it for the
+    // start of the next run of the event loop.
+    if(m_initializingBackends==0) {
+        m_allInitializedEmitter->start();
+    } else {
         slotFetching();
+    }
 }
 
 void ResourcesModel::callerFetchingChanged()
@@ -147,7 +160,7 @@ void ResourcesModel::callerFetchingChanged()
     } else {
         m_initializingBackends--;
         if(m_initializingBackends==0)
-            emit allInitialized();
+            m_allInitializedEmitter->start();
         else
             slotFetching();
     }
@@ -208,7 +221,7 @@ void ResourcesModel::registerAllBackends()
     const auto backends = f.allBackends();
     if(m_initializingBackends==0 && backends.isEmpty()) {
         qCWarning(LIBDISCOVER_LOG) << "Couldn't find any backends";
-        emit allInitialized();
+        m_allInitializedEmitter->start();
     } else {
         foreach(AbstractResourcesBackend* b, backends) {
             addResourcesBackend(b);
@@ -279,6 +292,7 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream*>& str
     for (auto stream: streams) {
         connect(stream, &ResultsStream::resourcesFound, this, &AggregatedResultsStream::addResults);
         connect(stream, &QObject::destroyed, this, &AggregatedResultsStream::destruction);
+        connect(this, &ResultsStream::fetchMore, stream, &ResultsStream::fetchMore);
         m_streams << stream;
     }
 
@@ -286,8 +300,15 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream*>& str
     connect(&m_delayedEmission, &QTimer::timeout, this, &AggregatedResultsStream::emitResults);
 }
 
+AggregatedResultsStream::~AggregatedResultsStream() = default;
+
 void AggregatedResultsStream::addResults(const QVector<AbstractResource *>& res)
 {
+    for(auto r : res)
+        connect(r, &QObject::destroyed, this, [this, r](){
+            m_results.removeAll(r);
+        });
+
     m_results += res;
 
     m_delayedEmission.start();
@@ -328,17 +349,6 @@ AggregatedResultsStream* ResourcesModel::search(const AbstractResourcesBackend::
     return new AggregatedResultsStream(streams);
 }
 
-AbstractResource* ResourcesModel::resourceForFile(const QUrl& file)
-{
-    AbstractResource* ret = nullptr;
-    foreach(auto backend, m_backends) {
-        ret = backend->resourceForFile(file);
-        if (ret)
-            break;
-    }
-    return ret;
-}
-
 void ResourcesModel::checkForUpdates()
 {
     for(auto backend: qAsConst(m_backends))
