@@ -23,6 +23,7 @@
 #include <QDebug>
 #include <QDBusInterface>
 #include <QDBusPendingReply>
+#include <QNetworkConfigurationManager>
 #include <KConfig>
 #include <KConfigGroup>
 #include <KRun>
@@ -30,9 +31,11 @@
 #include <KNotification>
 #include <KPluginFactory>
 
+#include "../libdiscover/utils.h"
+
 DiscoverNotifier::DiscoverNotifier(QObject * parent)
     : QObject(parent)
-    , m_verbose(false)
+    , m_manager(new QNetworkConfigurationManager(this))
 {
     configurationChanged();
 
@@ -43,7 +46,7 @@ DiscoverNotifier::DiscoverNotifier(QObject * parent)
             if (!m_needsReboot) {
                 m_needsReboot = true;
                 showRebootNotification();
-                Q_EMIT updatesChanged();
+                Q_EMIT stateChanged();
                 Q_EMIT needsRebootChanged(true);
             }
         });
@@ -54,6 +57,8 @@ DiscoverNotifier::DiscoverNotifier(QObject * parent)
     m_timer.setSingleShot(true);
     m_timer.setInterval(1000);
     updateStatusNotifier();
+
+    connect(m_manager, &QNetworkConfigurationManager::onlineStateChanged, this, &DiscoverNotifier::stateChanged);
 }
 
 DiscoverNotifier::~DiscoverNotifier() = default;
@@ -78,12 +83,12 @@ void DiscoverNotifier::showDiscoverUpdates()
 
 void DiscoverNotifier::showUpdatesNotification()
 {
-    if (state()==NoUpdates) {
+    if (state() != NormalUpdates && state() != SecurityUpdates) {
         //it's not very helpful to notify that everything is in order
         return;
     }
 
-    auto e = KNotification::event(QStringLiteral("Update"), message(), extendedMessage(), iconName(), nullptr, KNotification::CloseOnTimeout, QStringLiteral("discoverabstractnotifier"));
+    auto e = KNotification::event(QStringLiteral("Update"), message(), {}, iconName(), nullptr, KNotification::CloseOnTimeout, QStringLiteral("discoverabstractnotifier"));
     const QString name = i18n("Update");
     e->setDefaultAction(name);
     e->setActions({name});
@@ -92,33 +97,31 @@ void DiscoverNotifier::showUpdatesNotification()
 
 void DiscoverNotifier::updateStatusNotifier()
 {
-    uint securityCount = 0;
-    for (BackendNotifierModule* module: m_backends)
-        securityCount += module->securityUpdatesCount();
+    const bool hasSecurityUpdates = kContains(m_backends, [](BackendNotifierModule* module) { return module->hasSecurityUpdates(); });
+    const bool hasUpdates = hasSecurityUpdates || kContains(m_backends, [](BackendNotifierModule* module) { return module->hasUpdates(); });
 
-    uint count = securityCount;
-    foreach(BackendNotifierModule* module, m_backends)
-        count += module->updatesCount();
-
-    if (m_count == count && m_securityCount == securityCount)
+    if (m_hasUpdates == hasUpdates && m_hasSecurityUpdates == hasSecurityUpdates )
         return;
 
-    if (state() != NoUpdates && count > m_count) {
+    m_hasSecurityUpdates = hasSecurityUpdates;
+    m_hasUpdates = hasUpdates;
+
+    if (state() != NoUpdates) {
         m_timer.start();
     }
 
-    m_securityCount = securityCount;
-    m_count = count;
-    emit updatesChanged();
+    emit stateChanged();
 }
 
 DiscoverNotifier::State DiscoverNotifier::state() const
 {
     if (m_needsReboot)
         return RebootRequired;
-    else if (m_securityCount)
+    else if (!m_manager->isOnline())
+        return Offline;
+    else if (m_hasSecurityUpdates)
         return SecurityUpdates;
-    else if (m_count)
+    else if (m_hasUpdates)
         return NormalUpdates;
     else
         return NoUpdates;
@@ -135,6 +138,8 @@ QString DiscoverNotifier::iconName() const
             return QStringLiteral("update-none");
         case RebootRequired:
             return QStringLiteral("system-reboot");
+        case Offline:
+            return QStringLiteral("offline");
     }
     return QString();
 }
@@ -150,46 +155,18 @@ QString DiscoverNotifier::message() const
             return i18n("System up to date");
         case RebootRequired:
             return i18n("Computer needs to restart");
+        case Offline:
+            return i18n("Offline");
     }
     return QString();
 }
 
-QString DiscoverNotifier::extendedMessage() const
-{
-    if (m_count > 0 && m_securityCount > 0) {
-        QString allUpdates = i18ncp("First part of '%1, %2'",
-                                    "1 package to update", "%1 packages to update", m_count);
-
-        QString securityUpdates = i18ncp("Second part of '%1, %2'",
-                                         "of which 1 is security update", "of which %1 are security updates", m_securityCount);
-
-        return i18nc("%1 is '%1 packages to update' and %2 is 'of which %1 is security updates'",
-                     "%1, %2", allUpdates, securityUpdates);
-    } else if (m_count > 0) {
-        return i18np("1 package to update", "%1 packages to update", m_count);
-    } else if (m_securityCount > 0) {
-        return i18np("1 security update", "%1 security updates", m_securityCount);
-    } else {
-        return i18n("No packages to update");
-    }
-}
-
 void DiscoverNotifier::recheckSystemUpdateNeeded()
 {
     foreach(BackendNotifierModule* module, m_backends)
         module->recheckSystemUpdateNeeded();
 }
 
-uint DiscoverNotifier::securityUpdatesCount() const
-{
-    return m_securityCount;
-}
-
-uint DiscoverNotifier::updatesCount() const
-{
-    return m_count;
-}
-
 QStringList DiscoverNotifier::loadedModules() const
 {
     QStringList ret;
@@ -231,8 +208,3 @@ void DiscoverNotifier::foundUpgradeAction(UpgradeAction* action)
 
     notification->sendEvent();
 }
-
-bool DiscoverNotifier::isSystemUpToDate() const
-{
-    return m_count==0 && m_securityCount==0;
-}
